
;    Filename:	    Trap Timer.asm
;    Date:    		6-10-14
;
;	File Version: 1
;	Internal 4 MHz oscillator
;	32.768 KHz crystal driving TMR1
;*****************************************
;	see C:\MPASM MYFILE.ASM /PIC16F684                                                                 
;*****************************************


	LIST P=16F648A,r=hex,n=80,x=off,st=off 
 ; A HANDY LINE TO SPECIFY SOME DEFAULTS


	#include <P16F648A.inc>       ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_ON & DATA_CP_OFF & _LVP_OFF & _BODEN_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _INTOSC_OSC_NOCLKOUT 




;***** VARIABLE DEFINITIONS
;port_a	EQU	0x05
;port_c	EQU	0x07  ? portC ?
;CMCON0	EQU	0x19
;CMCON1	EQU	0x1A
;TRISC	EQU 0x87
;ANSEL	EQU 0x91
;WPUA	EQU 0x95
;OSCCON	EQU 0x8F

;  THE .INCLUDE FILE DEFINES ALL THE SYSTEM REGISTERS
;  THEY SHOULD NEVER BE REDIFINED LIKE YOUR 'port_a' 
;  THE OTHERS IN CAPS ARE JUST DUPLICATING THE .INC FILE SO NOT NEEDED

;  HAVE REPLACED ALL YOUR port a and c WITH THE CORRECT NAME
;  THE 648 DOES NOT HAVE A PORTC 


;*****  USER   VARIABLE DEFINITIONS


	cblock	0x20	;start of general purpose registers
		count_a		;used in delay routine
		count_b		;switch setting
		boolean		;logic indicator bits
	endc




#define 	tmr0_of	INTCON, TMR0IF	;TMR0 counter overflow flag
#define 	tmr1_of	PIR1, TMR1IF	;TMR1 counter overflow flag

#define 	c_bit	STATUS, C
#define 	z_bit	STATUS, Z
#define 	tmr1_on	T1CON, 0	;control TMR1

#define 	pwr PORTA, 0	;maintain power to PIC
#define 	sol PORTA, 1	;drives the Solenoid or Relay
#define 	sw	PORTA, 2	;apply high to switch & LK1
#define 	LED PORTB, 5	;LED	
#define 	perm_high	boolean, 0 





	ORG     0x00

	call	Initialise
	bsf		pwr		;maintain power to PIC
	bsf		LED

	call	delay_5s	;5.05 sec LED on time (also allows the oscillator to start)
	bcf		LED		;LED off

	call	delay_500m	;Temp LC
	bsf		LED		;LED off - Temp LC
	call	delay_500m	;Temp LC
	bcf		LED		;LED off - Temp LC

	call	delay_500m	;Temp LC
	bsf		LED		;LED off - Temp LC
	call	delay_500m	;Temp LC
	bcf		LED		;LED off - Temp LC

	clrf	count_b
;end 5.1 sec delay *****************************************
	call	chk_sw	;determine the switch position
	movf	count_b, w
	btfsc	z_bit	;is count_b 0?
	goto	cce		;Yes - error switch open or set to 0
;************************************************************
	btfsc	perm_high	;is the sol output to be permanently high?
	bsf		sol			;yes
;perm_high is set if LK1 is inserted indicating that sol is to be high continuously for the
; timing duration. Otherwise sol will be pulsed high for 100 ms at the end of the timing period.

cca	call	delay_1h ;count_b = delay in hours
	decfsz	count_b, f		;count hours
	goto	cca

	bsf		sol		;operate solenoid
	call	delay_100m

ccb	clrf	T1CON	;stop TMR1 & oscillator
	clrf	INTCON	;cancel interrupts
	bcf		LED		;turn LED off
	clrf	PORTA	;turn power & sol off
	sleep
	NOP
;************************************************************
cce	;error case, all switch contacts open
	call	flash_LED
	goto	ccb

;sub routines ************************************************************
chk_sw
	bsf		sw
	movfw	PORTB
	movwf	count_b

	btfsc	count_b, 4	;is LK1 open or closed?
	bsf		perm_high	;open
	bcf		count_b, 4
	bcf		sw
	return

flash_LED
	movlw	0x06	;flash LED 6 times
	movwf	count_b		; approx 6 sec
ffa	bsf		LED
	call	delay_500m
	bcf		LED
	call	delay_500m
	decfsz	count_b, f
	goto	ffa
	return

delay_5s
	movlw	.77
	movwf	count_a	;5.14 sec
dda	bcf		tmr0_of
	btfss	tmr0_of
	goto	$-1

	decfsz	count_a, f
	goto	dda
	return

delay_100m			;102 ms
	movlw	0xFE
	movwf	TMR1H	;for a delay of 103.15 ms
	movlw	0x65
	movwf	TMR1L	;complement of 410 decimal
	movlw	0x01
	goto	delay

delay_500m			;509 ms
	movlw	0xF7
	movwf	TMR1H
	movlw	0xFD	;complement of 0x08 02 for 500 ms delay
	movwf	TMR1L
	movlw	0x01

delay	;One TMR1 loop is 16 Sec (256 * 256 * 8/32768)
		; since the Xtal frequency is 32768 Hz
;need TMR1 driven by a 32768 Hz xtal - to use sleep
	bcf		T1CON,0  ;stop TMR1
	movwf	count_a
	bcf		tmr1_of	;clear interrupt flag
	bsf		T1CON,0  ;start TMR1
cla	btfss	tmr1_of	;await TMR1 overflow
	goto	$-1		;no
	bcf		tmr1_of	;clear interrupt flag
	decfsz	count_a, f
	goto	cla
	return

delay_1h			;stopped at 3591 sec
	movlw	.221	;=> delay of 3600 sec = 1 Hr
	movwf	count_a	;required time value in count_a
	clrf	TMR1H
	clrf	TMR1L	;to prevent possible overflow soon after entering sleep	
	bsf		T1CON,0  ;start TMR1
clb	bcf		tmr1_of	;clear interrupt flag

	btfss	tmr1_of	;Temp LC
	goto	$-1

;	sleep		;await TMR1 overflow
;	nop
	decfsz	count_a, f
	goto	clb
	return



Initialise
	banksel	PORTA		; select memory bank 0
	clrf	PORTA
	clrf	PORTB		; outputs low

	movlw	0x07		; set GP<2:0> to
;	movwf	CMCON0		; digital
;	clrf	CMCON1

	movwf	CMCON		; digital
	clrf	CMCON
; 	THERE IS ONLY CMCON ON THE 648 

	BANKSEL	INTCON
; 	MISSED BANKSEL  

	bsf		INTCON, PEIE	;enable peripheral interrupts
	bsf		INTCON, 5	;enable TMR0 interrupt

	banksel	PIE1		; select memory bank 1
	bsf		PIE1, TMR1IE	;enable TMR1 interrupt
;	clrf	WPUA		; weak pull up register, pullups disabled

; 	WRONG NAME, BY DEFUALT TURNED OFF

	movlw	B'00111000'	; bit 3 must always = 1
	movwf	TRISA		; port data direction register


;	movlw	B'00011111'	; bit 3 must always = 1


;  THE REAL CAUSE OF YOUR STIMULUS PROBLEM  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;  HERE YOUR HAVE DEFINED PORT BITS 0 TO 4 AS INPUTS AND BITS 5,6,7 AS OUTPUTS.
;  BITS 6 AND 7 ARE TIMER 1 INPUTS - CONFLICT !
;  NEW VALUE  --
		
	MOVLW	B'11011111' 

;	movwf	TRISC		; port data direction register

;   NO PORTC ON 648

	movwf	TRISB		; port data direction register

	movlw	B'10000111'	; portA pullups disabled, TMR0 prescale 256
	movwf	OPTION_REG

;	clrf	OSCCON
; 	clrf	ANSEL		; digital i/o

;	NO ANSEL OR OSCON ON 648


	banksel	T1CON	;select memory bank 0

;	movlw	0x32	;see spec page 50

	MOVLW	0X3F    ; BITS 2 AND 3 TO ON  BIT 3 

	movwf	T1CON	;TMR1 - 1:8 prescale, 32.768 KHz clock
	return

	end

